/** \file interface.cpp
 * Implements functions and classes for interfacing with OpenGL and GLUT
 * \author Robert W Martin
 * \date 03-16-2007
 */

/**
 * Include Files
 */
#include "interface.h"

/** namespace Interface
 * Namespace that Contains all of the "Stuff" for OpenGL and GLUT
 */
namespace Interface {

	/**
	 * Implementation for class OpenGL
	 */
	OpenGL *OpenGL::m_pSingleton = NULL;

	OpenGL::OpenGL( void ) {
		m_iWindow = -1;
	}
	OpenGL::~OpenGL( void ) {
		Shutdown();
	}
	OpenGL *OpenGL::GetSingleton( void ) {
		if( m_pSingleton == NULL ) {
			m_pSingleton = new OpenGL();
		}
		return m_pSingleton;
	}
	void OpenGL::KillSingleton( void ) {
		if( m_pSingleton != NULL ) {
			delete m_pSingleton;
			m_pSingleton = NULL;
		}
	}
	void OpenGL::Initialize( int argc, char **argv, int iWidth, int iHeight, char *pTitle ) {

		if( m_iWindow != -1 ) return;
		glutInit( &argc, argv );
		glutInitWindowPosition( 100, 100 );
		glutInitWindowSize( iWidth, iHeight );
		glutInitDisplayMode( GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB );
		m_iWindow = glutCreateWindow( pTitle );
		Viewport( iWidth, iHeight );
		glDisable( GL_DEPTH_TEST );
		glBlendFunc( GL_SRC_ALPHA, GL_SRC_ALPHA );

	}
	void OpenGL::Shutdown( void ) {
		if( m_iWindow == -1 ) return;
		glutDestroyWindow( m_iWindow );
		m_iWindow = -1;
	}
	void OpenGL::SetRenderer( DrawFunc draw, ResizeFunc resize ) {
		glutDisplayFunc( draw );
		glutReshapeFunc( resize );
		resize( m_iWidth, m_iHeight );
	}
	void OpenGL::SetInput( KboardFunc kboard, MouseFunc mouse ) {
		glutKeyboardFunc( kboard );
		glutMouseFunc( mouse );
	}
	void OpenGL::StartMainLoop( void ) {
		glutMainLoop();
	}
	void OpenGL::Viewport( int iW, int iH ) {
		glViewport( 0, 0, iW, iH );
		m_iWidth = iW;
		m_iHeight = iH;
	}
	void OpenGL::PostRedisplay( void ) {
		glutPostRedisplay();
	}
	void OpenGL::SwapBuffers( void ) {
		glFlush();
		glutSwapBuffers();
	}
	void OpenGL::Blend( bool bBlend ) {
		if( bBlend ) glEnable( GL_BLEND );
		else glDisable( GL_BLEND );
	}
	void OpenGL::Clear( bool bPixel, bool bDepth ) {
		int bits = 0;
		if( bPixel ) bits |= GL_COLOR_BUFFER_BIT;
		if( bDepth ) bits |= GL_DEPTH_BUFFER_BIT;
		glClear( bits );
	}
	void OpenGL::Begin( int iMode ) {
		glBegin( iMode );
	}
	void OpenGL::End( void ) {
		glEnd();
	}
	void OpenGL::Flush( void ) {
		glFlush();
	}
	void OpenGL::Vertex( float x, float y, float z ) {
		glVertex3f( (GLfloat)x, (GLfloat)y, (GLfloat)z );
	}
	void OpenGL::String( char *pString, float x, float y ) {

		glMatrixMode( GL_PROJECTION );
		glPushMatrix();
		glLoadIdentity();
		gluOrtho2D( 0, m_iWidth, 0, m_iHeight );
		glScalef( 1, -1, 1 );
		glTranslatef( 0, -m_iHeight, 0 );

		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();

		glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
		glRasterPos2f( x, y );

		char *c = pString;
		while( *c ) {
			glutBitmapCharacter( GLUT_BITMAP_HELVETICA_18, *c );
			++c;
		}

		glPopMatrix();

		glMatrixMode(GL_PROJECTION);
		glPopMatrix();

		glMatrixMode(GL_MODELVIEW);

	}
	void OpenGL::Mode( int iMode ) {
		glMatrixMode( iMode );
	}
	void OpenGL::Identity( void ) {
		glLoadIdentity();
	}
	void OpenGL::Ortho( float fLeft, float fRight, float fTop, float fBottom ) {
		glOrtho( (GLdouble)fLeft, (GLdouble)fRight, (GLdouble)fBottom, (GLdouble)fTop, 0.1, 100.0 );
	}
	void OpenGL::Translate( float x, float y, float z ) {
		glTranslatef( (GLfloat)x, (GLfloat)y, (GLfloat)z );
	}
	void OpenGL::Push( void ) {
		glPushMatrix();
	}
	void OpenGL::Pop( void ) {
		glPopMatrix();
	}
	int OpenGL::Width( void ) {
		return m_iWidth;
	}
	int OpenGL::Height( void ) {
		return m_iHeight;
	}
	/**
	 * Implementation for class Color
	 */
	Color::Color( float r, float g, float b, float a ) {
		Set( r, g, b, a );
	}
	Color::~Color( void ) {}
	void Color::Set( float r, float g, float b, float a ) {
		m_R = r;
		m_G = g;
		m_B = b;
		m_A = a;
	}
	void Color::Activate( void ) {
		glColor4f( (GLfloat)m_R, (GLfloat)m_G, (GLfloat)m_B, (GLfloat)m_A );
	}

}// namespace Interface
