/** \file main.cpp
 * Main Entry Point file for RWM-Chess
 * \author Robert W Martin
 * \date 03-16-2007
 */

/**
 * Include Files
 */
#include "interface.h"
#include "game.h"
#include <iostream>

/**
 * Namespaces
 */
using namespace std;
using namespace Interface;
using namespace Game;

/**
 * Prototypes
 */
BoardSpace MousePosition( int iX, int iY );
void Render( void );
void Reshape( int iX, int iY );
void Mouse( int iButton, int iState, int iX, int iY );
void Keyboard( unsigned char cKey, int iX, int iY );

/**
 * Global Variables
 */
Board			g_Board;

int main( int argc, char **argv ) {

	OpenGL *pGL = OpenGL::GetSingleton();

	pGL->Initialize( argc, argv, InitialW, InitialH, "RWM-Chess v1.0.0" );
	pGL->SetRenderer( Render, Reshape );
	pGL->SetInput( Keyboard, Mouse );
	g_Board.Initialize();
	pGL->StartMainLoop();

	g_Board.Shutdown();
	OpenGL::KillSingleton();
	return 0;

}
BoardSpace MousePos( int iX, int iY ) {

	OpenGL *pGL = OpenGL::GetSingleton();

	// Re-center the Mouse Coordinates on [ 0, 0 ]
	float fX = iX - pGL->Width() / 2;
	float fY = iY - pGL->Height() / 2;

	// Scale the Mouse Coordinates by the Size of the Projection
	float xScale = ( ScreenR - ScreenL ) / (float)pGL->Width();
	float yScale = ( ScreenB - ScreenT ) / (float)pGL->Height();
	fX *= xScale;
	fY *= yScale;

	// Re-center the Mouse Coordinates on [ BoardStartX, BoardStartY ]
	fX -= BoardStartX;
	fY -= BoardStartY;

	// Offset by the Size of Board Spaces
	fX += SpaceSize / 2.0f;
	fY += SpaceSize / 2.0f;

	// Check for Out-of-bounds (Off-the-board)
	if( fX < 0.0f || fX > 2.0f * BoardSize || fY < 0.0f || fY > 2.0f * BoardSize ) {
		BoardSpace error = { -1, -1 };
		return error;
	}

	// Create the BoardSpace
	BoardSpace bs = { (int)fX / (int)SpaceSize, (int)fY / (int)SpaceSize, false };
	return bs;

}
void Render( void ) {

	OpenGL *pGL = OpenGL::GetSingleton();

	pGL->Clear( true, true );

	g_Board.Draw();

	pGL->Flush();
	pGL->SwapBuffers();

}
void Reshape( int iX, int iY ) {

	OpenGL *pGL = OpenGL::GetSingleton();

	pGL->Viewport( iX, iY );

	pGL->Mode( GL_PROJECTION );
	pGL->Identity();
	pGL->Ortho( ScreenL, ScreenR, ScreenT, ScreenB );

	pGL->Mode( GL_MODELVIEW );
	pGL->Identity();
	pGL->PostRedisplay();

}
void Mouse( int iButton, int iState, int iX, int iY ) {

	if( iButton == GLUT_LEFT_BUTTON && iState == GLUT_UP ) {

		g_Board.SelectPiece( MousePos( iX, iY ) );
		OpenGL::GetSingleton()->PostRedisplay();

	}
	if( iButton == GLUT_RIGHT_BUTTON && iState == GLUT_UP ) {

		g_Board.MovePiece( MousePos( iX, iY ) );
		OpenGL::GetSingleton()->PostRedisplay();

	}

}
void Keyboard( unsigned char cKey, int iX, int iY ) {

	if( cKey == 'r' ) {
		// Restart the Game...
		g_Board.Shutdown();
		g_Board.Initialize();
		OpenGL::GetSingleton()->PostRedisplay();
	}

}
